#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <string.h>
#include <time.h>

#include "skymapm.h"

/*	
*	NOTE: The jemx variable is 0 for JEM-X1 and 1 for JEM-X2 in these routines !!
*/

/* sky visibility across JEM-X detector surface */
/* October 2001. Version including interaction depth parameter, to accomodate high energy photons */
/* November 2002. Version includes:
   	1)  parameters and notes in the "skymap.aux" file
        2)  alfa, beta, gammav angles describing relative twisting of the mask, collimator and MS-plate
        3)  x and y offsets of the MS plate wrt the collimator
        4)  Invert order of X and Y in 'detindex' and 'skyindex' to be consistent with usage in JEM-X TM-data 
*/ 
/* December 2002. (skymaph.c) Version includes:
   Input of 4 smoothed shadowgrams "soft_shdg" based on real observations of empty fields.
   These shadowgrams are integrated over 4 energy bands: 3-9, 9-18, 18-36, 36-50 keV.
   "Dont-use" regions have been defined around the four calibration sources in the smoothed 
   shadowgrams, these are indicated by zeroes in the corresponding pixel positions
*/
/* January 2003. (skymapi.c) Version includes:
   Basic skymap version which will only carry out the raytracing part corresponding to skymaph.
   The derivation of model shadowgrams for the internal and diffuse background is moved to
   another, independent program "backmodel.c"
*/
/* August 2003. (skymapk.c) Version includes:
   The "missing holes" in the mask are taken into account (mask pattern is imported from njw-file: "maskhole.txt")
   The mask support structure is modelled and its shadows are included. (file: "strukdef.c")
*/
/* March 2004. (skymapl.c) Version includes:
   Consistent use of 'alignment.dat' relative to midi_b5
*/
/* May 2004. (skymapm.c) Version includes:
   New treatment of collimator dificiencies (tilt, curvature) via parameters in 'alignment.dat'
   Consistent use of 'alignment6.dat' relative to midi_b6
   August 2004: consistent use of 'alignment8.dat' relative to 'mb9'
*/

/* unsigned char struk[1100][1100]; */
/*	The "struk" array declared and defined inside the "strucdef.c" file
        contains information about the mask support structure and its shadowing of the mask. 
        The array covers a 550x550 mm2 area with pixels of 0.5x0.5 mm2.
        The access tests should be done in the following order:
        1) calculate intersection of ray with the mask plane. 
           Find corresponding pixel in the "struk" array.
        2) If the pixel value is 0 access is allowed along this path     (in open areas between structural members)
        3) If the pixel value is >= 32 access is forbidden along this path (hit solid structure or mask elim. hole)
        4) If the pixel value is >0 and <32:                (in potential shadow area close to a structural member)
           4a) If the pixel value is odd, 		             (close to a structural member of 21 mm height)
              Calculate intersection points of this photon with two leves at +/- 21.25 mm
           4b) If the pixel value is even, 		             (close to a structural member of 10 mm height)
              Calculate intersection points of this photon with two leves at +/- 10.25 mm
        5) If the pixel value in any of the two new points is 64 access is forbidden.         (hit solid structure)
        6) If the pixel value in any of the two new points is 32 
           			and the first pixel value was even, access is forbidden.      (hit solid structure)
        7) If the pixel value in both of the two new points is >= 0 
                                and the two values are different, access is forbidden.   (traverse solid structure)
        8) Otherwise access is allowed.

        The following symbols are used in the pixels of struk:
           0:	free access, no further tests are required.
           32:  access blocked by solid material in mask support structure of 10.25 mm height.
           64:  access blocked by solid material in mask support structure of 21.25 mm height.
           1, 3, 5, 9, 17: Photon close to high structure. 
              		  Test required at plus/minus 21.25 mm from the mask plane.
           2, 4, 6, 8: 16: Photon close to low structure. 
              		  Test required at plus/minus 10.25 mm from the mask plane.
        
*/
unsigned char d_allow[byt_max];	
/*      The d_allow array contains for each detector pixel a list of 
	sky directions which have access to this pixel. ("access" 
	here means that more half the pixel area is visible from 
	above the mask along the direction in question)
	
        The number of sky-bins in the list corresponding to a given 
        detector pixel is given as an entry in the d_allow_l
        array, the full 16-bit address of the first direction 
	having access to the pixel is given in the d_allow_p array.
        
        The list itself is organized as a difference list: each 
        direction defined by an 8-bit unsigned number giving the 
        difference between in addresses of the previous direction 
        and the current direction. Should two directions differ 
        by more than 254 units, the difference will be written by 
        N bytes with the value 255 and finally a byte with the 
	remaining difference (after subtraction of N*255) 

	(d_allow is generated by the skymap routine)
*/

unsigned d_allow_st[detdim];
/*	The d_allow_st array gives for each 16-bit pixel in the 
   	detector the 32-bit address of the corresponding list 
        in the d_allow array.
	(generated by the skymap routine)
*/

short d_allow_l[detdim];
/*	array giving the length of the list in the d_allow array
   	corresponding to a given detector pixel
        */
int d_allow_p[detdim];
/*	array giving the address of the first sky pixel visible 
   	from a given detector pixel 
*/

float soft_shdgr[4][detdim];
/*	Arrays holding information about 'dont-use' areas on the detector
*/

int illumin[4][skydim];
/*	these arrays gives the number of detector pixels illuminated from each sky pixel
        only high background regions eliminated
   	(one array for each energy band)
*/

short maskhole[22501];
/*	mask definition: 1: open element, 0: closed element 
*/

double parameters[20];	/* array with the specific  parameters for current skymap:
        parameters[0] = byt_index;	length of 'd_allow' list 
        parameters[1] = lsum;		sum of ('det_pixel'*'sky pixels visible from the det_pixel') 
        parameters[2] = isum;		number of active 'det_pixels'
        parameters[3] = skyxdim;	number of X-det_pixels in sky region of interest 
        parameters[5] = skyydim;	number of Y-det_pixels in sky region of interest 
        parameters[7] = cond;		condition parameter for inclusion of collimator in backprojection 
	          		cond = 0: disregard collimator in backprojection 
                                cond = 1: include collimator
        parameters[8] = jemx; 		JEM-X unit number 
        parameters[9] = FOV_radius	Radius of FOV expressed in sky-pixels (the unit used for skyxdim & skyydim)
           			Common-X is the direction from JEM-X2 towards JEM-X1 
        parameters[11] = alfa; 		angle between mask X-axis and Common-X 
        parameters[12] = beta;		angle between collimator X-axis and Common-X  		
        parameters[13] = gammav;	angle between microstrip X-axis and Common-X 

        parameters[15] = xshi;		X-shift of microstrip center relative to collimator center 
        parameters[16] = yshi;		Y-shift of microstrip center relative to collimator center 
        parameters[17] = maskoffs;	displacement of mask center spacecraft centerline, now not used
        parameters[18] = mask_height;	height of mask above detector window 
        parameters[19] = sky_version;	skymap version
*/

char notes[1000];	/* notes & comments for current skymap */
char sweethome[200];

int iorder;
int njwmask[200][200];
int nr, nx, ny, all_anod;

unsigned  byt_index, allowmax, sky_index;
unsigned loss[25], distri[100], gain[25];

   	/* begin parameters to be updated through alignment.dat */ 
double coll_pitch;
double coll_h;       /* height of collimator */
double coll_w;       /* halfthickness of lamellae */
double coll_r;	/* collimator radius */ 
double mask_r;	/* radius of coded mask pattern */
double mask_height;
double hexa_dim;     /* flat-to-flat dimension of the mask elements */
double struc_h1;
double struc_h2;
double alfa;
double beta;
double gammav;
double  xshi;
double  yshi;
double c_tiltx;
double c_tilty;
double coll_wb;
double coll_corner;

double  indep; 
int FOV_radius;
double coll_curv;
double mask_scale;
double x_rms[5]; 	/* position determination uncertainty at the boundaries*/
double rmsfact;
double y_rms[5]; 	/* the values given here are default values */

double scaleX;
double scaleY;
double phase2;
double  x2x;
double  y2y;
double  offX;
double  offY;
double  offQ;
double  offR; 
	/* end parameters to be updated through alignment.dat */ 


double collp30;		/* 30 times the collimator pitch */
double cosal, sinal, cosbe, sinbe, cosga, singa, coll_r2, c0_fact, mask_r2;
double r_sky, c_fact, c_factb, m_fact, rsin60, cot60, pi;
double s_fact_l, s_fact_h;
double alfarot, betarot, gammarot;	
double collp30top, coll_ptop;
FILE *ofil, *ref_bin;

void skymap(int jemx, int cond, double indep, int njwmask[200][200]);
void skymap_init(int jemx, double indep);
int alignment_init(int jemx);

/* ************************************************************* */

int main(int argc, char* argv[])
{
/*	
*	NOTE: The jemx variable is 0 for JEM-X1 and 1 for JEM-X2 in these routines !!
*/
   	int i, j, ii, normax, normin, bn, il_col[49], irsky, jemx, refsum;
        int normtal[20], kk, bi, normimin[4], normimax[4], cond;
        double normdist[20], normind[4][20], colnorm, xxsky, yysky, rrsky, rsky2;
        double indep;
        char skyfil[100], skofil[100], reffil[100];
        char signs[51] = { '0','1','2','3','4','5','6','7','8','9',
           		   'a','b','c','d','e','f','g','h','i','j',
                           'k','l','m','n','o','p','q','r','s','t',
                           'u','v','x','y','z','A','B','C','D','E',
                           'F','G','H','I','J','K','L','M','N','O','P'};
        char sky_version = 'm';	  /* NOTE: this letter identifies the version of the skymap program 
                                     and the generated datafiles */
        
        time_t t;
        
        FOV_radius = FOV_rad;

        sprintf(sweethome, "/home/nl/jemx/c_soft");
        
        cond = 1;		/* include collimator obstructions as default */
        jemx = 1;		/* take JEM-X2 as default */
        indep = 10.0;		/* interaction depth of photons in the detector */
        alfarot = 0.0; 		/* adjustment angle between mask X-axis and Common-X */
        betarot = 0.0;		/* adjustment angle between collimator X-axis and Common-X */
        gammarot = 0.0;		/* adjustment angle between microstrip X-axis and Common-X */
        all_anod = 0;		/* eliminate bad anodes as default */
        
        if (argc > 1) jemx = atoi(argv[1]);
        else {
           printf(" jemx indep {all_anodes {0}}\n");
           return;
        }
        if (argc > 2) indep = atof(argv[2]);
        if (argc > 3) all_anod = atoi(argv[3]);

        jemx &= 1;
        if ((indep > 50.0) || (indep < 0.0)) {printf("Illegal indep-value\n"); return(0);}
        if ((cond > 1) || (cond < 0)) {printf("Illegal cond-value\n"); return(0);}

	if (cond == 1) sprintf(skyfil, "./skmap%1cc%1d_%1c", sky_version, jemx+1, signs[(int)(indep)]);
        if (cond == 0) sprintf(skyfil, "./skmap%1cn%1d_%1c", sky_version, jemx+1, signs[(int)(indep)]);

        strcpy(skofil, skyfil);
        strcat(skofil, ".txt");
        printf("%s\n", skofil);
        
        ofil = fopen(skofil, "wt");
        if (ofil == NULL) {printf(" Error in opening %s\n", skofil); exit(0);}
        
        time(&t);
        fprintf(ofil, "\n Calling skymap, byt_index: %10d,  time: %s", byt_index, ctime(&t));

        skymap(jemx, cond, indep, njwmask);
        
        time(&t);
	fprintf(ofil, "\n Returning from skymap, byt_index: %10d,  time: %s", byt_index, ctime(&t));
        fprintf(ofil, "\n Returning statistics:         Eliminated  Remaining"); 
        fprintf(ofil, "\n    0  Detector radius check:  %10u  %10u", loss[0], gain[0]);
        fprintf(ofil, "\n    1  Collim. top rad check:  %10u  %10u", loss[1], gain[1]);
        fprintf(ofil, "\n    2  Outside coded mask:     %10u  %10u", loss[2], gain[2]);
        fprintf(ofil, "\n    3  Detector yt>yd check:   %10u  %10u", loss[3], gain[3]);
        fprintf(ofil, "\n    4  Detector yt<yd check:   %10u  %10u", loss[4], gain[4]);
        fprintf(ofil, "\n    5  Detector dy<colw check: %10u  %10u", loss[5], gain[5]);
        fprintf(ofil, "\n    6  Detect dy>7-colw check: %10u  %10u", loss[6], gain[6]);
        fprintf(ofil, "\n    7  Detector xt>xd check:   %10u  %10u", loss[7], gain[7]);
        fprintf(ofil, "\n    8  Detector xt<xd check:   %10u  %10u", loss[8], gain[8]);
        fprintf(ofil, "\n    9  Detector dx<colw check: %10u  %10u", loss[9], gain[9]);
        fprintf(ofil, "\n    10 Detect dx>7-colw check: %10u  %10u", loss[10], gain[10]);
        fprintf(ofil, "\n    11 Blocked by ideal mask:  %10u  %10u", loss[11], gain[11]);
        fprintf(ofil, "\n    12 Bl. by eliminated hole: %10u  %10u", loss[12], gain[12]);
        fprintf(ofil, "\n    13 Blocked by structure:   %10u  %10u", loss[13], gain[13]);
        fprintf(ofil, "\n    15 Premature termination in backpro: %2d", loss[15]);
           
        printf("\n Returning from skymap, byt_index: %10u,  time: %s", byt_index, ctime(&t));
        printf("\n Returning statistics:         Eliminated  Remaining"); 
        printf("\n    0  Detector radius check:  %10u  %10u", loss[0], gain[0]);
        printf("\n    1  Collim. top rad check:  %10u  %10u", loss[1], gain[1]);
        printf("\n    2  Outside coded mask:     %10u  %10u", loss[2], gain[2]);
        printf("\n    3  Detector yt>yd check:   %10u  %10u", loss[3], gain[3]);
        printf("\n    4  Detector yt<yd check:   %10u  %10u", loss[4], gain[4]);
        printf("\n    5  Detector dy<colw check: %10u  %10u", loss[5], gain[5]);
        printf("\n    6  Detect dy>7-colw check: %10u  %10u", loss[6], gain[6]);
        printf("\n    7  Detector xt>xd check:   %10u  %10u", loss[7], gain[7]);
        printf("\n    8  Detector xt<xd check:   %10u  %10u", loss[8], gain[8]);
        printf("\n    9  Detector dx<colw check: %10u  %10u", loss[9], gain[9]);
        printf("\n    10 Detect dx>7-colw check: %10u  %10u", loss[10], gain[10]);
        printf("\n    11 Blocked by ideal mask:  %10u  %10u", loss[11], gain[11]);
        printf("\n    12 Bl. by eliminated hole: %10u  %10u", loss[12], gain[12]);
        printf("\n    13 Blocked by structure:   %10u  %10u", loss[13], gain[13]);
        printf("\n    15 Premature termination in backpro: %2d", loss[15]);
           
        fprintf(ofil, "\n\n Allowmax: %10u.   Distribution:   \n", allowmax);
        printf("\n\n Allowmax: %10u.   Distribution:   \n", allowmax);
           
        for (i=0; i<100; i++) {
           if ((i%10) == 0) fprintf(ofil, "\n");
           fprintf(ofil, " %6d", distri[i]);
           if ((i%10) == 0) printf("\n");
           printf(" %6d", distri[i]);
        }
        fprintf(ofil, "\n"); 
        printf("\n"); 
        fclose(ofil);

        time(&t);
	
        if (indep > 0)  {	/* only output 'skmap'-files if 'indep' > 0 */

           strcpy(skofil, skyfil);
           strcat(skofil, ".dat");
           printf("\n%s\n", skofil);

           time(&t);
           printf("\n Writing %s,   byt_index: %10u,  time: %s", skofil, byt_index, ctime(&t));
           ofil = fopen(skofil, "wb");
           if (ofil == NULL) {
              printf("Could not open %s, exiting!\n", skofil);
              return(0);
           }

           fwrite(d_allow, 1, byt_index, ofil);
           fclose(ofil);
           time(&t);
           printf("\n Completed writing %s,   byt_index: %10u,  time: %s\n", skofil, byt_index, ctime(&t));

           strcpy(skofil, skyfil);
           strcat(skofil, ".aux");
           printf("\n%s\n", skofil);
           ofil = fopen(skofil, "wb");
           if (ofil == NULL) {
              printf("Could not open %s, exiting!\n", skofil);
              return(0);
           }

           parameters[0] = byt_index;
           parameters[3] = skyxdim;
           parameters[4] = -FOV_radius;	/* was 'skyxoffs */
           parameters[5] = skyydim;
           parameters[6] = -FOV_radius;	/* was 'skyyoffs */
           parameters[7] = cond;
           parameters[8] = jemx; 
           parameters[9] = FOV_radius;
           parameters[19] = (float)(sky_version);
        
           fwrite(parameters, 8, 20, ofil);
           fwrite(d_allow_st, 4, detdim, ofil);
           fwrite(d_allow_l, 2, detdim, ofil);
           fwrite(d_allow_p, 4, detdim, ofil);
           fwrite(maskhole, 2, 22501, ofil);
           fwrite(notes, 1, 1000, ofil);
           fclose(ofil);

        } else { 	/* only output 'illumination' if 'indep' = 0 */
        
           if (all_anod == 0) sprintf(skofil, "detmap%1c-%1d.aux", sky_version, jemx+1);
           else sprintf(skofil, "detmap%1c+%1d.aux", sky_version, jemx+1);
           printf("%s\n", skofil);
           ofil = fopen(skofil, "wb");
           if (ofil == NULL) {
              printf("Could not open %s, exiting!\n", skofil);
              return(0);
           }
   
           fwrite(illumin, 4, 4*skydim, ofil);
           fclose(ofil);
   
           if (all_anod == 0) sprintf(skofil, "illum_%1c_%1d.txt", sky_version, jemx+1);
           else sprintf(skofil, "illum_%1c_%1dp.txt", sky_version, jemx+1);
           printf("%s\n\n", skofil);
           ofil = fopen(skofil, "wt");
           if (ofil == NULL) {
              printf("Could not open %s, exiting!\n", skofil);
              return(0);
           }
   
           for (i=0; i<skydim; i++) {
              if ((i % skyxdim) == 0) fprintf(ofil, "\n%4d", illumin[1][i]);
              else fprintf(ofil, " %4d", illumin[1][i]);
           }
           fprintf(ofil, "\n");
           fclose(ofil);
        
        }
        
	return(0);
}

/* *************************************************************** */

void skymap(int jemx, int cond, double indep, int njwmask[200][200])	
   /* sky visibility across JEM-X detector surface */
/*	
*	Niels Lund, February 2001	Version 0.1
*	Niels Lund, November 2002	Version 3.0
*	Niels Lund, August 2003		Version 4.0
*
*	April 2001	Version 1.0 (skyxdim x skyydim sky pixels)
*	November 2002   Version 3.0 Detector units are mm. 
*       August 2003     Version 4.0 Introduces structure shadowing and "eliminated holes" in mask pattern
*	Danish Space Research Institute
*	
*	NOTE: The jemx variable is 0 for JEM-X1 and 1 for JEM-X2 in these routines !!
*
*	The XY-coordinate system used in this routine is aligned with 
*	the detector collimator lamellae. The microstrip pattern is 
*	approximately aligned with the lamellae. The cathode positions 
*	are (nearly) along the X-axis, the backside positions nearly 
*	along the Y-axis. The center of the detector is located at
*	position (128, 128). The central collimator crossing is
*	also at this position. The  center of the collimator falls at 
*       the corner between 4 pixel boundaries.
*
*	The unit used for the X- and Y-positions on the detector are 
*	1 mm. Actually, the X- and Y-integer positions are only 
*	approximately separated by 1 mm. A detailed map, specifying 
*   	the float values of the X- and Y-position of the four corners 
*	of each pixel have been prepared by Carl Budtz-J. 
*
*	The full field of view of the JEM-X detector is defined by the
*	dimensions of the mask (535 mm diameter) and detector (250 mm
*	diameter) and by the mask-detector separation (about 3400 mm).
*
*	An orthogonal direction grid of 'skyxdim'*'skyydim' sky pixels 
*       is defined within the full field of view, which has a radius 
*	of 'FOV_radius' pixels. 
*
*/
{
   int i, j, k, m, allownum, n, ring, nn, dif, sky2, iq, iv, ky, mx, mmm, njwhole, strblock;
   int xy7, x20, y20, xy20, looplim, xcdif, ycdif, xyacc, pfl=0, p_loss, allowpix=0;
   int zz20[skyxdim], njw1, njw2, C255 = 255*255, idy, jdx, ksy, msx;
   unsigned det_index, dd_index, sky_index;
   unsigned mx2,  xd7, yd7, xc7, yc7, cur_index, aind;
   unsigned st_byt_in;
   unsigned char ch255;

   double ic, jc, xdet, ydet, xmask, ymask;
   double allow, rm2;
   double ydet2, qmask, vmask, dif7, xcoll2;
   double xdmod7, ydmod7, xtmod7, ytmod7, dxlow, dylow, dxhigh, dyhigh, xtemp;

   double ycolbot, ycoltop, xcolbot, xcoltop, xrmask, yrmask, xmask2;
   double xrcbot, xrctop, yrcbot, yrctop, xrdet, yrdet;
   /* initializations */

   skymap_init(jemx, indep);
   
   /* Common-X is the direction from JEM-X2 towards JEM-X1 */
   
   printf("\n msk_rot: %+5.2lf,  col_rot: %+5.2lf,  ms_rot %5.2lf, x/y_shft: %+4.2lf/%4.2lf",
      (double)(alfa * 180.0 / pi), (double)(beta * 180.0 / pi), (double)(gammav * 180.0 / pi), xshi, yshi);
   fprintf(ofil, "\n msk_rot: %+5.2lf,  col_rot: %+5.2lf,  ms_rot %5.2lf, x/y_shft: %+4.2lf/%4.2lf",
      (double)(alfa * 180.0 / pi), (double)(beta * 180.0 / pi), (double)(gammav * 180.0 / pi), xshi, yshi);

   parameters[10] = indep;
   parameters[11] = alfa; 
   parameters[12] = beta;
   parameters[13] = gammav; 

   parameters[15] = xshi; 
   parameters[16] = yshi;
   parameters[18] = mask_height;

   fprintf(ofil, "\n c_fact: %10.6lf, c_factb: %10.6lf,  m_fact: %10.6lf", c_fact, c_factb, m_fact);
   printf("\n c_fact: %10.6lf, c_factb: %10.6lf, m_fact: %10.6lf\n", c_fact, c_factb, m_fact);
   
   ch255 = 255;

   for (i=0; i<detdim; i++) d_allow_l[i] = 0;

   for (i=0; i<skyxdim; i++) {
      j = (i + 8) / 20;
      if (j < 1) j = 1;
      if (j > 12) j = 12;
      zz20[i] = j - 1;
   }
      
   for (i=0; i<25; i++) loss[i] = gain[i] = 0;

   det_index = 0;
   sky_index = 0;
   cur_index = 0;
   allownum = 0;
   byt_index = 0;
   allowmax = 0;

    /* ***************  main loop ********************* */

   for (idy=0; idy<256; idy++) {		/* loop over detector Y */

      p_loss = 0;

      yrdet = (double)(idy) - 127.5 + yshi;
      ydet2 = yrdet * yrdet;

      if ((idy%8) == 0) {
         printf("\n Det_radius  Coll_rad.    Mask_rad.    +Y cross  -Y cross  +Y thick  -Y thick");
         printf("  Ideal_Mask  Elimin_hole  Struct  Accept\n");
         for (mmm=0; mmm<7; mmm++) {
            printf(" %10u", loss[mmm]);		/* print loss counters for every 8th Y-value */
         }
         printf(" %9u", loss[11]);
         printf(" %9u", loss[12]);
         printf(" %9u\n", loss[13]);
         for (mmm=0; mmm<7; mmm++) {
            printf(" %10u", gain[mmm]);		/* print gain counters for every 8th Y-value */
         }
         printf(" %9u", gain[11]);
         printf(" %9u", gain[12]);
         printf(" %9u", gain[13]);
         printf(" %9u\n", gain[13]);
      }

      for (jdx=0; jdx<256; jdx++) {		/* loop over detector X */
         
	 if (allownum > 0) {		/* save length information for previous list */
		d_allow_l[det_index] = allownum;
                aind = allownum / 100;
                distri[aind]++;
                if (allownum > allowmax) allowmax = allownum;
		allownum = 0;
	 }

	 det_index = jdx * 256 + idy;
	 d_allow_st[det_index] = byt_index;
         d_allow_l[det_index] = 0;
         
	 xrdet = (double)(jdx) - 127.5 + xshi;
         if ((ydet2 + xrdet*xrdet) > coll_r2) {		/* detector radius check */
             loss[0] += skydim;
             p_loss = 99;
             continue;	   
         } else gain[0] += skydim; 
         
         xdet = xrdet * cosga - yrdet * singa;
         ydet = yrdet * cosga + xrdet * singa;
         
	 /* xdet and ydet are coordinates of a detector pixel in a coord. system aligned after Common-X */

         xyacc = 0;
         
         for (msx=0; msx<skyxdim; msx++) {			/* loop over sky-direction X */
            mx = msx - FOV_radius;
            mx2 = mx * mx;

            xcolbot = (double)(mx) * c_factb + xdet;
            xcoltop = (double)(mx) * c_fact + xdet;
            xcoll2 = xcoltop * xcoltop;
            
            xrmask = (double)(mx) * m_fact + xdet;

            xmask2 = xrmask * xrmask;

            dxlow = (double)(mx) * s_fact_l;
            dxhigh = (double)(mx) * s_fact_h;
                
            for (ksy=0; ksy<skyydim; ksy++) {			/* loop over sky-direction Y */
            ky = ksy - FOV_radius;
                
            p_loss = 0;

               ycolbot = (double)(ky) * c_factb + ydet;
               ycoltop = (double)(ky) * c_fact + ydet;
            
                if ((xcoll2 + ycoltop * ycoltop) > coll_r2) {
                   loss[1] += 1;				/* Collimator top radius check */
                   p_loss = 1;
                   continue;		
                } else gain[1] += 1; 
                
                yrmask = (double)(ky) * m_fact + ydet;
   		rm2 = yrmask * yrmask + xmask2;
                
                dylow = (double)(ky) * s_fact_l;
                dyhigh = (double)(ky) * s_fact_h;
                
		if (rm2 > mask_r2) {			/* the event falls outside the coded mask */
                   loss[2]++;
                   p_loss = 2;
                   continue;
                } else gain[2]++; 

                xrcbot = xcolbot * cosbe + ycolbot * sinbe;	/* rotation through -beta! */
                yrcbot = ycolbot * cosbe - xcolbot * sinbe;
            
                xrctop = xcoltop * cosbe + ycoltop * sinbe;	/* rotation through -beta! */
                yrctop = ycoltop * cosbe - xcoltop * sinbe;
            
                /* determine open directions of collimator  
                   the collimator lamellae are located on positions:
                   -126.9,-119.85, ... ,-21.15, -14.1, -7.05, 0, 7.05, 14.1, 21.15, ... , 119.85, 126.9
                   both in the X- and Y-direction 
                */

                yd7 = (int)((yrcbot + collp30) / coll_pitch);	/* determine bottom collimator hole index
                   							   collp30 >detector radius */
                yc7 = (int)((yrctop + c_tilty + collp30top) / coll_ptop);	/* determine top collimator hole index */
	    
                if (cond) {
                   if (yc7 > yd7) {
                      /* the event is lost due to wall crossing in the collimator */
                      loss[3]++;					/* Y-collimator wall +crossing check */
                      if (pfl) {
                         if (p_loss == 3) printf("3"); 
                         else printf("\n%1d  Y-wall cross+  %3d ", pfl, ksy);
                      }
                      p_loss = 3;
                      continue;
                   } else gain[3]++;  
                   
                   if (yc7 < yd7) {
                      /* the event is lost due to wall crossing in the collimator */
                      loss[4]++;					/* Y-collimator wall -crossing check */
                      if (pfl) {
                         if (p_loss == 4) printf("4"); 
                         else printf("\n%1d  Y-wall cross-  %3d ", pfl, ksy);
                      }
                      p_loss = 4;
                      continue;
                   } else gain[4]++; 	   
                   
                   ydmod7 = yrcbot + collp30 - coll_pitch * yd7;	/* get remainder Ycoord within this hole */
                   if (ydmod7 < coll_wb) {
                      /* the event is lost at the bottom edge of the collimator */
                      loss[5]++;					/* Y-collimator wall -thickness check */
                      if (pfl) {
                         if (p_loss == 5) printf("5"); 
                         else printf("\n%1d  Y-bot edge -  %3d ", pfl, ksy);
                      }
                      p_loss = 5;
                      continue;
                   } 
                   	   
                   if (ydmod7 > (coll_pitch - coll_wb)) {
                      /* the event is lost at the bottom edge of the collimator */
                      if (pfl) {
                         if (p_loss == 6) printf("6"); 
                         else printf("\n%1d  Y-bot edge +  %3d ", pfl, ksy);
                      }
                      loss[6]++;					/* Y-collimator wall +thickness check */
                      p_loss = 6;
                      continue;
                   } 

                   ytmod7 = yrctop + c_tilty + collp30top - coll_ptop * yc7;	/* get remainder Ycoord within this hole */
                   if (ytmod7 < coll_w) {
                      /* the event is lost at the top edge of the collimator */
                      if (pfl) {
                         if (p_loss == 15) printf("V"); 
                         else printf("\n%1d  Y-top edge -  %3d ", pfl, ksy);
                      }
                      loss[5]++;					/* Y-collimator wall -thickness check */
                      p_loss = 15;
                      continue;
                   } else gain[5]++; 
                   	   
                   if (ytmod7 > (coll_ptop - coll_w)) {
                      /* the event is los  at the top edge of the collimator */
                      if (pfl) {
                         if (p_loss == 16) printf("W"); 
                         else printf("\n%1d  Y-top edge -  %3d ", pfl, ksy);
                      }
                      loss[6]++;					/* Y-collimator wall +thickness check */
                      p_loss = 16;
                      continue;
                   } else gain[6]++; 
                }	   

 	        xd7 = (int)((xrcbot + collp30) / coll_pitch);
                xc7 = (int)((xrctop + c_tiltx + collp30top) / coll_ptop);
                    
                if (cond) {
                    if (xc7 > xd7) {
                       /* the event is lost due to wall crossing in the collimator */
                       if (pfl) {
                          if (p_loss == 7) printf("7"); 
                          else printf("\n%1d  Y-wall cross+  %3d ", pfl, ksy);
                       }
                       loss[7]++;
                       p_loss = 7;
                       continue;
                    } else gain[7]++; 	   
                    
                    if (xc7 < xd7) {
                       /* the event is lost due to wall crossing in the collimator */
                       if (pfl) {
                          if (p_loss == 8) printf("8"); 
                          else printf("\n%1d  Y-wall cross-  %3d ", pfl, ksy);
                       }
                       loss[8]++;
                       p_loss = 8;
                       continue;
                    } else gain[8]++; 

                    xdmod7 = xrcbot + collp30 - coll_pitch * xd7;
                    if (xdmod7 < coll_w) {
                       /* the event is lost at the bottom edge of the collimator */
                       if (pfl) {
                          if (p_loss == 9) printf("9"); 
                          else printf("\n%1d  Y-bot edge -  %3d ", pfl, ksy);
                       }
                       loss[9]++;
                       p_loss = 9;
                       continue;
                    } 	   

                    if (xdmod7 > (coll_pitch - coll_wb)) {
                       /* the event is lost at the bottom edge of the collimator */
                       if (pfl) {
                          if (p_loss == 10) printf("a"); 
                          else printf("\n%1d  Y-bot edge +  %3d ", pfl, ksy);
                       }
                       loss[10]++;
                       p_loss = 10;
                       continue;
                    } 	   

                    xtmod7 = xrctop + c_tiltx + collp30top - coll_ptop * xc7;
                    if (xtmod7 < coll_w) {
                       /* the event is lost at the top edge of the collimator */
                       if (pfl) {
                          if (p_loss == 19) printf("?"); 
                          else printf("\n%1d  Y-top edge -  %3d ", pfl, ksy);
                       }
                       loss[9]++;
                       p_loss = 19;
                       continue;
                    } else gain[9]++; 	   

                    if (xtmod7 > (coll_pitch - coll_w)) {
                       /* the event is lost at the top edge of the collimator */
                       if (pfl) {
                          if (p_loss == 110) printf("A"); 
                          else printf("\n%1d  Y-top edge +  %3d ", pfl, ksy);
                       }
                       loss[10]++;
                       p_loss = 110;
                       continue;
                    } else gain[10]++; 	   
                }
                
                xmask = xrmask * cosal + yrmask * sinal; 
                ymask = yrmask * cosal - xrmask * sinal;
                
		sky_index = msx * skyydim + ksy;		

                qmask = xmask / hexa_dim - ymask * cot60;
                vmask = ymask * rsin60;
                
                iq = qmask + 22501.5;
                iv = vmask + 22501.5;
                nn = (iq + 1133 * iv) % 22501;
                
                iq = iq - 22501;
                iv = iv - 22501;
                
                njw1 = 99+iq+iv;
                njw2 = 100+iv;
                njwhole = 0;
                if ((njw1 >= 0) && (njw1 < 200) && (njw2 >= 0) && (njw2 < 200)) {
                   if (njwmask[njw1][njw2] == 6) njwhole = 1;
                } else printf("Index error for jemxmask: %3d %3d\n", njw1, njw2);
                
                
		if ((int)(maskhole[nn]) > 0) { 
                   gain[11]++;
                   strblock = struchk(xmask, ymask, xdet, ydet, 0);
                   if (njwhole) {
                      gain[12]++;
                      if (strblock == 0) { 
                         gain[13]++;

                         if (indep == 0) {                         
                            if (soft_shdgr[0][det_index]  > 0.0) illumin[0][sky_index]++;
                            if (soft_shdgr[1][det_index]  > 0.0) illumin[1][sky_index]++;
                            if (soft_shdgr[2][det_index]  > 0.0) illumin[2][sky_index]++;
                            if (soft_shdgr[3][det_index]  > 0.0) illumin[3][sky_index]++;
                         }
                         
                         dif = 0;
                         if (allownum |= 0) {
			    dif = sky_index - cur_index;
			    while (dif > 254) {
                               d_allow[byt_index] = ch255;
                               byt_index++;
                               if (byt_index > byt_max) {
                                  printf("\n byt_index exceeded range! idy,jdx,ksy,msx: %4d %4d %4d %4d",
                                     idy,jdx,ksy,msx);
                                  fprintf(ofil, "\n byt_index exceeded range! idy,jdx,ksy,msx: %4d %4d %4d %4d",
                                     idy,jdx,ksy,msx);
                                  return;
                               }
                               dif -= 255;
	 		    } 
                            d_allow[byt_index] = (unsigned char)(dif);
                            byt_index++;
                            if (byt_index > byt_max) {
                               printf("\n byt_index exceeded range! idy,jdx,ksy,msx: %4d %4d %4d %4d",
                                  idy,jdx,ksy,msx);
                               fprintf(ofil, "\n byt_index exceeded range! idy,jdx,ksy,msx: %4d %4d %4d %4d",
                                  idy,jdx,ksy,msx);
                               return;
                            }
			 } else {
                            d_allow_p[det_index] = sky_index;
                         }
			 cur_index = sky_index;
			 allownum++;   
                         if (pfl) {
                            if ((p_loss == 111) || (p_loss == 11)) printf("B"); 
                            else printf("\n%1d Open hole! %3d ", pfl, ksy);
                            allowpix++;
                         }
                         p_loss = 111;
                      } else {
                         loss[13]++;
                      }
                   } else {
                      loss[12]++;	/* nl mask open, njw mask closed (eliminated hole) */ 
                      if (strblock == 0) loss[14]++;	
                         /* eliminated hole and no structure blocking, suspicious case! */ 
                   }
		} else {
                   if (pfl) {
                      if ((p_loss == 111) || (p_loss == 11)) printf("b");  
                      else printf("\n%1d Closedhole %3d ", pfl, ksy);
                   }
                   loss[11]++;
                   p_loss = 11;
                }

            }
         }
      }
   }
   struchk(0.0, 0.0, 0.0, 0.0, 2);

}

/* **************************************************************** */

void skymap_init(int jemx, double indepc)	
 
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                                               */        
/*      skymap_init.c read a list parameters to be used in the generation        */        
/*      of skymaps.                                                              */
/*                                                                               */
/*      Niels Lund, March 2004                                                   */        
/*      Uses NJW-mask and detector support struct.                               */
/*                                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
*
*	The full field of view of the JEM-X detector is defined by the
*	dimensions of the mask (535 mm diameter) and detector (250 mm
*	diameter) and by the mask-detector separation (about 3400 mm).
*
*       The full field of view, which has a radius of 'FOV_radius' pixels. 
*
*/

{
   int i, j, k, m, allownum, n, ring, nn, dif, sky2, iq, iv, ky, mx, mmm, iqp, ivp;
   int x20, y20, xy20, looplim, xcdif, ycdif, xyacc, p_loss, allowpix=0, njw1, njw2;
   int njwhole, strblock, njwOK, p_count=0, ntal, ma, deadanode_num, status;
   int deadanod[256], deadanod_type[256], deadanod_num, iy, ee;

   unsigned det_index, dd_index, sky_index;
   unsigned mx2,  xd7, yd7, xc7, yc7, cur_index, aind;
   unsigned st_byt_in;

   double ic, jc, xdet, ydet, xmask, ymask;
   double ydet2, qmask, vmask, dif7, xcoll2, rm2, allow;
   double xdmod7, ydmod7, xtmod7, ytmod7, sum, xtemp;

   double ycolbot, ycoltop, xcolbot, xcoltop, xrmask, yrmask, xmask2;
   double xrcbot, xrctop, yrcbot, yrctop, xrdet, yrdet;
   float norm, indat[2];
   
   static int first = 1;

   char mhline[410], *charpt, pchar, maskfil[200], alfil[200], iline[128];
   
   FILE *mhfil;
   
   /* initializations */

   if (first) {

      first = 0;

      pi = 3.14159;
      twopi = pi * 2.0;
      gtorad = pi / 180.0;

      sprintf(maskfil, "%s/reference/maskhole.txt", sweethome);
/*      printf("Opening: %s", maskfil);
*/
      mhfil = fopen(maskfil, "rt");
      if (mhfil == NULL) {printf("Could not open maskhole.txt\n"); exit(0);}
      
      for (i=0; i<200; i++) {	/* read NJW definition of open mask holes */
         fgets(mhline, 410, mhfil);
         for (j=0; j<200; j+=20) {
            charpt = &mhline[j*2];
            sscanf(charpt, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
               &njwmask[i][j+0], &njwmask[i][j+1], &njwmask[i][j+2], &njwmask[i][j+3], 
               &njwmask[i][j+4], &njwmask[i][j+5], &njwmask[i][j+6], &njwmask[i][j+7], 
               &njwmask[i][j+8], &njwmask[i][j+9], &njwmask[i][j+10], &njwmask[i][j+11], 
               &njwmask[i][j+12], &njwmask[i][j+13], &njwmask[i][j+14], &njwmask[i][j+15], 
               &njwmask[i][j+16], &njwmask[i][j+17], &njwmask[i][j+18], &njwmask[i][j+19]);
         }
      }
      fclose(mhfil);

      iorder = 22501;	/* 4u + 1 */
      nr  = 1133;

      for (m=1; m<=iorder; m++) maskhole[m] = 0;
      for (m=1; m<=iorder-1; m++) {
		ntal = (m * m) % iorder;
		ma = ntal;
		ntal = (int)((ma * ma) % iorder);
                maskhole[ntal] = 1;
      }
      for (m=1; m<=iorder; m++) if ((int)(maskhole[m]) > 0) n++;
      printf("Holes in nlmask: %5d\n", n);

      status = alignment_init(jemx);

      if (status < 1) {printf("Bad or missing 'alignment8.dat', exiting!\n"); exit(0);}

      if (FOV_radius != FOV_rad) {printf("alignment data not matching FOV_rad, exiting!\n"); exit(0);}

      /* Common-X is the direction from JEM-X2 towards JEM-X1 */
  
      collp30 = coll_pitch * 30.0;  /* 30 time the collimator pitch, used in the "modulo" calc */

      coll_ptop = coll_pitch * coll_curv;  
      collp30top = coll_ptop * 30.0;  
   
      coll_r2 = coll_r * coll_r;
      mask_r2 = mask_r * mask_r;

      cosal = cos(alfa);
      sinal = sin(alfa);
      cosbe = cos(beta);
      sinbe = sin(beta);
      cosga = cos(gammav);
      singa = sin(gammav);
   
      r_sky = mask_r + coll_r;	/* field of view radius */

      c0_fact = 6.6 / FOV_radius; 	/* stepsize (mm) in skymap at collimator top level */ 

      m_fact = c0_fact * (mask_height + indepc) / coll_h;	/* corresponding stepsize (mm) at mask level */	
      c_factb = c0_fact * indepc / coll_h;			/* corresponding stepsize (mm) at bottom of collimator */
      c_fact = c0_fact * (coll_h + indepc) / coll_h;	/* corresponding stepsize (mm) at top of collimator */
   
      s_fact_l = c0_fact * struc_h1 * 2.0 / coll_h; /* corresp. delta-step for low structure members (0.5 mm step) */	
      s_fact_h = c0_fact * struc_h2 * 2.0 / coll_h; /* corresp. delta-step for high structure members (0.5 mm step) */

      cot60 = 1.0 / (hexa_dim * 1.73205081);
      rsin60 = 2.0 * cot60;

      for (ee=0; ee<4; ee++)
         for (i=0; i<skydim; i++) illumin[ee][i] = 0;

      if (all_anod == 0) sprintf(alfil, "%s/reference/period5new/smooth%1d-anod.bin", sweethome, jemx+1);
      else sprintf(alfil, "%s/reference/period5new/smooth%1d+anod.bin", sweethome, jemx+1);

      mhfil = fopen(alfil, "rb");
      if (mhfil == NULL) {
         printf("File not found! (%s)\n", alfil);
         exit(0);
      } 
      fread(soft_shdgr, 4, 4*detdim, mhfil);
      fclose(mhfil);

      printf("Initialization completed, skyxdim: %4d,  skyydim: %4d, skydim: %6d\n", skyxdim, skyydim, skydim);
   }
   return;
}

/* ***************************************************************************************** */

int alignment_init(int jemx)
{
      FILE *mhfil;
      char alfil[200], mhline[200];
      float indat[2];
      int i, n;
      
      sprintf(alfil, "alignment8.dat");
      mhfil = fopen(alfil, "rt");
      if (mhfil != NULL) goto al_sta;

      sprintf(alfil, "%s/reference/alignment8.dat", sweethome);
      mhfil = fopen(alfil, "rt");
      if (mhfil == NULL) {
         printf("No alignment file found! (%s)\n", alfil);
         return(0);;
      } else {
al_sta:  for (i=0; i<200; i++) {
            fgets(mhline, 200, mhfil);
            if (feof(mhfil)) goto q_end;
            sscanf(mhline, "%d", &n);
            if (n == 1) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_pitch = indat[jemx];}
            if (n == 2) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_h = indat[jemx];}
            if (n == 3) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_w = indat[jemx];}
            if (n == 4) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_r = indat[jemx];}
            if (n == 5) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_r = indat[jemx];}
            if (n == 6) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_height = indat[jemx];}
            if (n == 7) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); hexa_dim = indat[jemx];}
            if (n == 8) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); struc_h2 = indat[jemx];}
            if (n == 9) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); struc_h1 = indat[jemx];}
            if (n == 10) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); alfa = indat[jemx];}
            if (n == 11) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); beta = indat[jemx];}
            if (n == 12) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); gammav = indat[jemx];}
            if (n == 13) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); xshi = indat[jemx];}
            if (n == 14) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); yshi = indat[jemx];}
            if (n == 15) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); c_tiltx = indat[jemx];}
            if (n == 16) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); c_tilty = indat[jemx];}

            if (n == 17) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_wb = indat[jemx];}
            if (n == 18) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_corner = indat[jemx];}

/*            if (n == 20) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); indep = indat[jemx];} */
            if (n == 21) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); FOV_radius = indat[jemx];}
            if (n == 22) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_curv = indat[jemx];}
            if (n == 23) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_scale = indat[jemx];}
            
            if (n == 30) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[0] = indat[jemx];}
            if (n == 31) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[1] = indat[jemx];}
            if (n == 32) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[2] = indat[jemx];}
            if (n == 33) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[3] = indat[jemx];}
            if (n == 34) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[4] = indat[jemx];}
            if (n == 35) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); rmsfact = indat[jemx];}

            if (n == 40) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[0] = indat[jemx];}
            if (n == 41) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[1] = indat[jemx];}
            if (n == 42) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[2] = indat[jemx];}
            if (n == 43) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[3] = indat[jemx];}
            if (n == 44) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[4] = indat[jemx];}

            if (n == 100) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); scaleX = indat[jemx];}
            if (n == 101) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); scaleY = indat[jemx];}
            if (n == 102) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); phase2 = indat[jemx];}
            if (n == 103) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x2x = indat[jemx];}
            if (n == 104) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y2y = indat[jemx];}
            if (n == 105) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); offX = indat[jemx];}
            if (n == 106) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); offY = indat[jemx];}
            if (n == 107) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); offQ = indat[jemx];}
            if (n == 108) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); offR = indat[jemx];}
         }
      }
q_end:

      for (i=0; i<5; i++) {x_rms[i] *= rmsfact; y_rms[i] *= rmsfact;}

      hexa_dim *= mask_scale;
      mask_height *= mask_scale;

      coll_wb += coll_w;
      alfa *= gtorad;
      beta *= gtorad;
      gammav *= gtorad;
      
      printf("\nRead parameters: %lf %lf %lf %lf %lf %lf %lf %lf\n", 
         coll_pitch, coll_h, coll_w, coll_r, mask_r, hexa_dim, struc_h2, struc_h1);
      printf("Read parameters: %lf %lf %lf %lf %lf %lf %f %f %lf %lf\n", 
         alfa, beta, gammav, xshi, yshi, mask_height, coll_wb, rmsfact);
      fclose(mhfil);

      
   return(1);
}
/* *************************************************************** */

